"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetadataConverter = void 0;
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const path_1 = require("path");
const graceful_fs_1 = require("graceful-fs");
const fileSystemHandler_1 = require("../utils/fileSystemHandler");
const errors_1 = require("../errors");
const collections_1 = require("../collections");
const registry_1 = require("../registry");
const streams_1 = require("./streams");
class MetadataConverter {
    constructor(registry = new registry_1.RegistryAccess()) {
        this.registry = registry;
    }
    async convert(comps, targetFormat, output) {
        var _a;
        try {
            const cs = comps instanceof collections_1.ComponentSet ? comps : new collections_1.ComponentSet(comps, this.registry);
            const components = (comps instanceof collections_1.ComponentSet ? Array.from(comps.getSourceComponents()) : comps).filter((comp) => comp.type.isAddressable !== false);
            let manifestContents;
            const isSource = targetFormat === 'source';
            const tasks = [];
            let writer;
            let mergeSet;
            let packagePath;
            let defaultDirectory;
            switch (output.type) {
                case 'directory':
                    if (output.packageName) {
                        cs.fullName = output.packageName;
                    }
                    manifestContents = cs.getPackageXml();
                    packagePath = this.getPackagePath(output);
                    defaultDirectory = packagePath;
                    writer = new streams_1.StandardWriter(packagePath);
                    if (!isSource) {
                        const manifestPath = (0, path_1.join)(packagePath, MetadataConverter.PACKAGE_XML_FILE);
                        tasks.push(graceful_fs_1.promises.writeFile(manifestPath, manifestContents));
                        // For deploying destructive changes
                        const destructiveChangesTypes = cs.getTypesOfDestructiveChanges();
                        if (destructiveChangesTypes.length) {
                            // for each of the destructive changes in the component set, convert and write the correct metadata
                            // to each manifest
                            destructiveChangesTypes.map((destructiveChangesType) => {
                                const file = this.getDestructiveManifest(destructiveChangesType);
                                const destructiveManifestContents = cs.getPackageXml(4, destructiveChangesType);
                                const destructiveManifestPath = (0, path_1.join)(packagePath, file);
                                tasks.push(graceful_fs_1.promises.writeFile(destructiveManifestPath, destructiveManifestContents));
                            });
                        }
                    }
                    break;
                case 'zip':
                    if (output.packageName) {
                        cs.fullName = output.packageName;
                    }
                    manifestContents = cs.getPackageXml();
                    packagePath = this.getPackagePath(output);
                    defaultDirectory = packagePath;
                    writer = new streams_1.ZipWriter(packagePath);
                    if (!isSource) {
                        writer.addToZip(manifestContents, MetadataConverter.PACKAGE_XML_FILE);
                        // For deploying destructive changes
                        const destructiveChangesTypes = cs.getTypesOfDestructiveChanges();
                        if (destructiveChangesTypes.length) {
                            // for each of the destructive changes in the component set, convert and write the correct metadata
                            // to each manifest
                            destructiveChangesTypes.map((destructiveChangeType) => {
                                const file = this.getDestructiveManifest(destructiveChangeType);
                                const destructiveManifestContents = cs.getPackageXml(4, destructiveChangeType);
                                writer.addToZip(destructiveManifestContents, file);
                            });
                        }
                    }
                    break;
                case 'merge':
                    if (!isSource) {
                        throw new errors_1.LibraryError('error_merge_metadata_target_unsupported');
                    }
                    defaultDirectory = output.defaultDirectory;
                    mergeSet = new collections_1.ComponentSet();
                    // since child components are composed in metadata format, we need to merge using the parent
                    for (const component of output.mergeWith) {
                        mergeSet.add((_a = component.parent) !== null && _a !== void 0 ? _a : component);
                    }
                    writer = new streams_1.StandardWriter(output.defaultDirectory);
                    writer.forceIgnoredPaths = output.forceIgnoredPaths;
                    break;
            }
            const conversionPipeline = (0, streams_1.pipeline)(new streams_1.ComponentReader(components), new streams_1.ComponentConverter(targetFormat, this.registry, mergeSet, defaultDirectory), writer);
            tasks.push(conversionPipeline);
            await Promise.all(tasks);
            const result = { packagePath };
            if (output.type === 'zip' && !packagePath) {
                result.zipBuffer = writer.buffer;
            }
            else if (output.type !== 'zip') {
                result.converted = writer.converted;
            }
            return result;
        }
        catch (e) {
            throw new errors_1.ConversionError(e);
        }
    }
    getPackagePath(outputConfig) {
        let packagePath;
        const { genUniqueDir = true, outputDirectory, packageName, type } = outputConfig;
        if (outputDirectory) {
            if (packageName) {
                packagePath = (0, path_1.join)(outputDirectory, packageName);
            }
            else {
                if (genUniqueDir) {
                    packagePath = (0, path_1.join)(outputDirectory, `${MetadataConverter.DEFAULT_PACKAGE_PREFIX}_${Date.now()}`);
                }
                else {
                    packagePath = (0, path_1.normalize)(outputDirectory);
                }
            }
            if (type === 'zip') {
                packagePath += '.zip';
                (0, fileSystemHandler_1.ensureDirectoryExists)((0, path_1.dirname)(packagePath));
            }
            else {
                (0, fileSystemHandler_1.ensureDirectoryExists)(packagePath);
            }
        }
        return packagePath;
    }
    getDestructiveManifest(destructiveChangesType) {
        if (destructiveChangesType === collections_1.DestructiveChangesType.POST) {
            return MetadataConverter.DESTRUCTIVE_CHANGES_POST_XML_FILE;
        }
        else if (destructiveChangesType === collections_1.DestructiveChangesType.PRE) {
            return MetadataConverter.DESTRUCTIVE_CHANGES_PRE_XML_FILE;
        }
    }
}
exports.MetadataConverter = MetadataConverter;
MetadataConverter.PACKAGE_XML_FILE = 'package.xml';
MetadataConverter.DESTRUCTIVE_CHANGES_POST_XML_FILE = 'destructiveChangesPost.xml';
MetadataConverter.DESTRUCTIVE_CHANGES_PRE_XML_FILE = 'destructiveChangesPre.xml';
MetadataConverter.DEFAULT_PACKAGE_PREFIX = 'metadataPackage';
//# sourceMappingURL=metadataConverter.js.map